@@ -30,12 +30,19 @@
 
 GST_DEBUG_CATEGORY_STATIC (gst_omx_buffer_pool_debug_category);
 #define GST_CAT_DEFAULT gst_omx_buffer_pool_debug_category
-GST_DEBUG_CATEGORY_STATIC (CAT_PERFORMANCE);
 
 typedef struct _GstOMXMemory GstOMXMemory;
 typedef struct _GstOMXMemoryAllocator GstOMXMemoryAllocator;
 typedef struct _GstOMXMemoryAllocatorClass GstOMXMemoryAllocatorClass;
 
+enum
+{
+  SIG_ALLOCATE,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 struct _GstOMXMemory
 {
   GstMemory mem;
@@ -171,11 +178,14 @@ gst_omx_memory_allocator_alloc (GstAllocator * allocator, GstMemoryFlags flags,
  * pool to the OMX port or provide the OMX buffers directly to other
  * elements.
  *
- *
- * A buffer is in the pool if it is currently owned by the port,
- * i.e. after OMX_{Fill,Empty}ThisBuffer(). A buffer is outside
+ * An output buffer is in the pool if it is currently owned by the port,
+ * i.e. after OMX_FillThisBuffer(). An output buffer is outside
  * the pool after it was taken from the port after it was handled
- * by the port, i.e. {Empty,Fill}BufferDone.
+ * by the port, i.e. FillBufferDone.
+ *
+ * An input buffer is in the pool if it is currently available to be filled
+ * upstream. It will be put back into the pool when it has been processed by
+ * OMX, (EmptyBufferDone).
  *
  * Buffers can be allocated by us (OMX_AllocateBuffer()) or allocated
  * by someone else and (temporarily) passed to this pool
@@ -188,7 +198,7 @@ gst_omx_memory_allocator_alloc (GstAllocator * allocator, GstMemoryFlags flags,
  * They correspond 1:1 to the OMX buffers of the port, which are allocated
  * before the pool is started.
  *
- * Acquiring a buffer from this pool happens after the OMX buffer has
+ * Acquiring an output buffer from this pool happens after the OMX buffer has
  * been acquired from the port. gst_buffer_pool_acquire_buffer() is
  * supposed to return the buffer that corresponds to the OMX buffer.
  *
@@ -203,8 +213,6 @@ gst_omx_memory_allocator_alloc (GstAllocator * allocator, GstMemoryFlags flags,
  * back to the component (OMX_FillThisBuffer()) when it is released.
  */
 
-static GQuark gst_omx_buffer_data_quark = 0;
-
 #define DEBUG_INIT \
   GST_DEBUG_CATEGORY_INIT (gst_omx_buffer_pool_debug_category, "omxbufferpool", 0, \
       "debug category for gst-omx buffer pool base class");
@@ -219,6 +227,9 @@ static gboolean
 gst_omx_buffer_pool_start (GstBufferPool * bpool)
 {
   GstOMXBufferPool *pool = GST_OMX_BUFFER_POOL (bpool);
+  gboolean has_buffers;
+  GstStructure *config;
+  guint min, max;
 
   /* Only allow to start the pool if we still are attached
    * to a component and port */
@@ -227,8 +238,39 @@ gst_omx_buffer_pool_start (GstBufferPool * bpool)
     GST_OBJECT_UNLOCK (pool);
     return FALSE;
   }
+
+  pool->port->using_pool = TRUE;
+
+  has_buffers = (pool->port->buffers != NULL);
   GST_OBJECT_UNLOCK (pool);
 
+  config = gst_buffer_pool_get_config (bpool);
+  gst_buffer_pool_config_get_params (config, NULL, NULL, &min, &max);
+  gst_structure_free (config);
+  if (max > min) {
+    GST_WARNING_OBJECT (bpool,
+        "max (%d) cannot be higher than min (%d) as pool cannot allocate buffers on the fly",
+        max, min);
+    return FALSE;
+  }
+
+  if (!has_buffers) {
+    gboolean result = FALSE;
+
+    GST_DEBUG_OBJECT (bpool, "Buffers not yet allocated on port %d of %s",
+        pool->port->index, pool->component->name);
+
+    g_signal_emit (pool, signals[SIG_ALLOCATE], 0, &result);
+
+    if (!result) {
+      GST_WARNING_OBJECT (bpool,
+          "Element failed to allocate buffers, can't start pool");
+      return FALSE;
+    }
+  }
+
+  g_assert (pool->port->buffers);
+
   return
       GST_BUFFER_POOL_CLASS (gst_omx_buffer_pool_parent_class)->start (bpool);
 }
@@ -239,21 +281,31 @@ gst_omx_buffer_pool_stop (GstBufferPool * bpool)
   GstOMXBufferPool *pool = GST_OMX_BUFFER_POOL (bpool);
   gint i = 0;
 
-  /* When not using the default GstBufferPool::GstAtomicQueue then
-   * GstBufferPool::free_buffer is not called while stopping the pool
-   * (because the queue is empty) */
-  for (i = 0; i < pool->buffers->len; i++)
-    GST_BUFFER_POOL_CLASS (gst_omx_buffer_pool_parent_class)->release_buffer
-        (bpool, g_ptr_array_index (pool->buffers, i));
+  if (pool->buffers) {
+    /* When not using the default GstBufferPool::GstAtomicQueue then
+     * GstBufferPool::free_buffer is not called while stopping the pool
+     * (because the queue is empty) */
+    for (i = 0; i < pool->buffers->len; i++)
+      GST_BUFFER_POOL_CLASS (gst_omx_buffer_pool_parent_class)->release_buffer
+          (bpool, g_ptr_array_index (pool->buffers, i));
+
+    /* Remove any buffers that are there */
+    g_ptr_array_set_size (pool->buffers, 0);
+  }
 
   /* Remove any buffers that are there */
   g_ptr_array_set_size (pool->buffers, 0);
 
+  GST_DEBUG_OBJECT (pool, "deallocate OMX buffers");
+  gst_omx_port_deallocate_buffers (pool->port);
+
   if (pool->caps)
     gst_caps_unref (pool->caps);
   pool->caps = NULL;
 
   pool->add_videometa = FALSE;
+  pool->deactivated = TRUE;
+  pool->port->using_pool = TRUE;
 
   return GST_BUFFER_POOL_CLASS (gst_omx_buffer_pool_parent_class)->stop (bpool);
 }
@@ -283,10 +335,11 @@ gst_omx_buffer_pool_set_config (GstBufferPool * bpool, GstStructure * config)
 {
   GstOMXBufferPool *pool = GST_OMX_BUFFER_POOL (bpool);
   GstCaps *caps;
+  guint size, min;
 
   GST_OBJECT_LOCK (pool);
 
-  if (!gst_buffer_pool_config_get_params (config, &caps, NULL, NULL, NULL))
+  if (!gst_buffer_pool_config_get_params (config, &caps, &size, &min, NULL))
     goto wrong_config;
 
   if (caps == NULL)
@@ -313,6 +366,9 @@ gst_omx_buffer_pool_set_config (GstBufferPool * bpool, GstStructure * config)
     gst_caps_unref (pool->caps);
   pool->caps = gst_caps_ref (caps);
 
+  /* Ensure max=min as the pool won't be able to allocate more buffers while active */
+  gst_buffer_pool_config_set_params (config, caps, size, min, min);
+
   GST_OBJECT_UNLOCK (pool);
 
   return GST_BUFFER_POOL_CLASS (gst_omx_buffer_pool_parent_class)->set_config
@@ -348,8 +404,6 @@ gst_omx_buffer_pool_alloc_buffer (GstBufferPool * bpool,
   GstBuffer *buf;
   GstOMXBuffer *omx_buf;
 
-  g_return_val_if_fail (pool->allocating, GST_FLOW_ERROR);
-
   omx_buf = g_ptr_array_index (pool->port->buffers, pool->current_buffer_index);
   g_return_val_if_fail (omx_buf != NULL, GST_FLOW_ERROR);
 
@@ -465,7 +519,7 @@ gst_omx_buffer_pool_alloc_buffer (GstBufferPool * bpool,
 
       for (i = 0; i < GST_VIDEO_INFO_N_PLANES (&pool->video_info); i++) {
         if (info.stride[i] != stride[i] || info.offset[i] != offset[i]) {
-          GST_CAT_DEBUG_OBJECT (CAT_PERFORMANCE, pool,
+          GST_DEBUG_OBJECT (pool,
               "Need to copy output frames because of stride/offset mismatch: plane %d stride %d (expected: %d) offset %"
               G_GSIZE_FORMAT " (expected: %" G_GSIZE_FORMAT
               ") nStride: %d nSliceHeight: %d ", i, stride[i], info.stride[i],
@@ -491,8 +545,7 @@ gst_omx_buffer_pool_alloc_buffer (GstBufferPool * bpool,
     }
   }
 
-  gst_mini_object_set_qdata (GST_MINI_OBJECT_CAST (buf),
-      gst_omx_buffer_data_quark, omx_buf, NULL);
+  gst_omx_buffer_set_omx_buf (buf, omx_buf);
 
   *buffer = buf;
 
@@ -514,13 +567,27 @@ gst_omx_buffer_pool_free_buffer (GstBufferPool * bpool, GstBuffer * buffer)
   }
   GST_OBJECT_UNLOCK (pool);
 
-  gst_mini_object_set_qdata (GST_MINI_OBJECT_CAST (buffer),
-      gst_omx_buffer_data_quark, NULL, NULL);
+  gst_omx_buffer_set_omx_buf (buffer, NULL);
 
   GST_BUFFER_POOL_CLASS (gst_omx_buffer_pool_parent_class)->free_buffer (bpool,
       buffer);
 }
 
+static GstBuffer *
+find_buffer_from_omx_buffer (GstOMXBufferPool * pool, GstOMXBuffer * omx_buf)
+{
+  guint i;
+
+  for (i = 0; i < pool->buffers->len; i++) {
+    GstBuffer *buf = g_ptr_array_index (pool->buffers, i);
+
+    if (gst_omx_buffer_get_omx_buf (buf) == omx_buf)
+      return buf;
+  }
+
+  return NULL;
+}
+
 static GstFlowReturn
 gst_omx_buffer_pool_acquire_buffer (GstBufferPool * bpool,
     GstBuffer ** buffer, GstBufferPoolAcquireParams * params)
@@ -544,9 +611,7 @@ gst_omx_buffer_pool_acquire_buffer (GstBufferPool * bpool,
       GstOMXBuffer *omx_buf;
 
       if (pool->output_mode == GST_OMX_BUFFER_MODE_DMABUF) {
-        omx_buf =
-            gst_mini_object_get_qdata (GST_MINI_OBJECT_CAST (buf),
-            gst_omx_buffer_data_quark);
+        omx_buf = gst_omx_buffer_get_omx_buf (buf);
       } else {
         g_assert (mem
             && g_strcmp0 (mem->allocator->mem_type, GST_OMX_MEMORY_TYPE) == 0);
@@ -560,9 +625,23 @@ gst_omx_buffer_pool_acquire_buffer (GstBufferPool * bpool,
     }
   } else {
     /* Acquire any buffer that is available to be filled by upstream */
-    ret =
-        GST_BUFFER_POOL_CLASS (gst_omx_buffer_pool_parent_class)->acquire_buffer
-        (bpool, buffer, params);
+    GstOMXBuffer *omx_buf;
+    GstOMXAcquireBufferReturn r;
+    GstOMXWait wait = GST_OMX_WAIT;
+
+    if (params && (params->flags & GST_BUFFER_POOL_ACQUIRE_FLAG_DONTWAIT))
+      wait = GST_OMX_DONT_WAIT;
+
+    r = gst_omx_port_acquire_buffer (pool->port, &omx_buf, wait);
+    if (r == GST_OMX_ACQUIRE_BUFFER_OK) {
+      *buffer = find_buffer_from_omx_buffer (pool, omx_buf);
+      g_return_val_if_fail (*buffer, GST_FLOW_ERROR);
+      return GST_FLOW_OK;
+    } else if (r == GST_OMX_ACQUIRE_BUFFER_FLUSHING) {
+      return GST_FLOW_FLUSHING;
+    } else {
+      return GST_FLOW_ERROR;
+    }
   }
 
   return ret;
@@ -577,11 +656,10 @@ gst_omx_buffer_pool_release_buffer (GstBufferPool * bpool, GstBuffer * buffer)
 
   g_assert (pool->component && pool->port);
 
-  if (!pool->allocating && !pool->deactivated) {
-    omx_buf =
-        gst_mini_object_get_qdata (GST_MINI_OBJECT_CAST (buffer),
-        gst_omx_buffer_data_quark);
-    if (pool->port->port_def.eDir == OMX_DirOutput && !omx_buf->used) {
+  if (gst_buffer_pool_is_active (bpool)) {
+    omx_buf = gst_omx_buffer_get_omx_buf (buffer);
+    if (pool->port->port_def.eDir == OMX_DirOutput && !omx_buf->used &&
+        !pool->deactivated) {
       /* Release back to the port, can be filled again */
       err = gst_omx_port_release_buffer (pool->port, omx_buf);
       if (err != OMX_ErrorNone) {
@@ -589,24 +667,8 @@ gst_omx_buffer_pool_release_buffer (GstBufferPool * bpool, GstBuffer * buffer)
             ("Failed to relase output buffer to component: %s (0x%08x)",
                 gst_omx_error_to_string (err), err));
       }
-    } else if (!omx_buf->used) {
-      /* TODO: Implement.
-       *
-       * If not used (i.e. was not passed to the component) this should do
-       * the same as EmptyBufferDone.
-       * If it is used (i.e. was passed to the component) this should do
-       * nothing until EmptyBufferDone.
-       *
-       * EmptyBufferDone should release the buffer to the pool so it can
-       * be allocated again
-       *
-       * Needs something to call back here in EmptyBufferDone, like keeping
-       * a ref on the buffer in GstOMXBuffer until EmptyBufferDone... which
-       * would ensure that the buffer is always unused when this is called.
-       */
-      g_assert_not_reached ();
-      GST_BUFFER_POOL_CLASS (gst_omx_buffer_pool_parent_class)->release_buffer
-          (bpool, buffer);
+    } else if (pool->port->port_def.eDir == OMX_DirInput) {
+      gst_omx_port_requeue_buffer (pool->port, omx_buf);
     }
   }
 }
@@ -636,6 +698,8 @@ gst_omx_buffer_pool_finalize (GObject * object)
     gst_caps_unref (pool->caps);
   pool->caps = NULL;
 
+  g_clear_pointer (&pool->component, gst_omx_component_unref);
+
   G_OBJECT_CLASS (gst_omx_buffer_pool_parent_class)->finalize (object);
 }
 
@@ -645,8 +709,6 @@ gst_omx_buffer_pool_class_init (GstOMXBufferPoolClass * klass)
   GObjectClass *gobject_class = (GObjectClass *) klass;
   GstBufferPoolClass *gstbufferpool_class = (GstBufferPoolClass *) klass;
 
-  gst_omx_buffer_data_quark = g_quark_from_static_string ("GstOMXBufferData");
-
   gobject_class->finalize = gst_omx_buffer_pool_finalize;
   gstbufferpool_class->start = gst_omx_buffer_pool_start;
   gstbufferpool_class->stop = gst_omx_buffer_pool_stop;
@@ -657,7 +719,9 @@ gst_omx_buffer_pool_class_init (GstOMXBufferPoolClass * klass)
   gstbufferpool_class->acquire_buffer = gst_omx_buffer_pool_acquire_buffer;
   gstbufferpool_class->release_buffer = gst_omx_buffer_pool_release_buffer;
 
-  GST_DEBUG_CATEGORY_GET (CAT_PERFORMANCE, "GST_PERFORMANCE");
+  signals[SIG_ALLOCATE] = g_signal_new ("allocate",
+      G_TYPE_FROM_CLASS (gobject_class), G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
+      G_TYPE_BOOLEAN, 0);
 }
 
 static void
@@ -674,7 +738,7 @@ gst_omx_buffer_pool_new (GstElement * element, GstOMXComponent * component,
 
   pool = g_object_new (gst_omx_buffer_pool_get_type (), NULL);
   pool->element = gst_object_ref (element);
-  pool->component = component;
+  pool->component = gst_omx_component_ref (component);
   pool->port = port;
   pool->output_mode = output_mode;
 
