@@ -220,7 +220,7 @@ gst_omx_h264_enc_class_init (GstOMXH264EncClass * klass)
 
   gst_element_class_set_static_metadata (element_class,
       "OpenMAX H.264 Video Encoder",
-      "Codec/Encoder/Video",
+      "Codec/Encoder/Video/Hardware",
       "Encode H.264 video streams",
       "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
 
@@ -676,19 +676,6 @@ gst_omx_h264_enc_set_format (GstOMXVideoEnc * enc, GstOMXPort * port,
     gst_caps_unref (peercaps);
   }
 
-  /* Change default profile to high-10 if input is 10 bits */
-  if (profile == OMX_VIDEO_AVCProfileMax) {
-    GstVideoFormat format;
-
-    format = state->info.finfo->format;
-    if (format == GST_VIDEO_FORMAT_NV12_10LE32 ||
-        format == GST_VIDEO_FORMAT_NV16_10LE32) {
-      GST_DEBUG_OBJECT (self,
-          "Set profile to high-10 as input is a 10 bits format");
-      profile = OMX_VIDEO_AVCProfileHigh10;
-    }
-  }
-
   if (profile != OMX_VIDEO_AVCProfileMax || level != OMX_VIDEO_AVCLevelMax) {
     /* OMX provides 2 API to set the profile and level. We try using the
      * generic one here and the H264 specific when calling
@@ -737,32 +724,11 @@ gst_omx_h264_enc_get_caps (GstOMXVideoEnc * enc, GstOMXPort * port,
       "alignment", G_TYPE_STRING, "au", NULL);
 
   if (err == OMX_ErrorNone) {
-    switch (param.eProfile) {
-      case OMX_VIDEO_AVCProfileBaseline:
-        profile = "baseline";
-        break;
-      case OMX_VIDEO_AVCProfileMain:
-        profile = "main";
-        break;
-      case OMX_VIDEO_AVCProfileExtended:
-        profile = "extended";
-        break;
-      case OMX_VIDEO_AVCProfileHigh:
-        profile = "high";
-        break;
-      case OMX_VIDEO_AVCProfileHigh10:
-        profile = "high-10";
-        break;
-      case OMX_VIDEO_AVCProfileHigh422:
-        profile = "high-4:2:2";
-        break;
-      case OMX_VIDEO_AVCProfileHigh444:
-        profile = "high-4:4:4";
-        break;
-      default:
-        g_assert_not_reached ();
-        gst_caps_unref (caps);
-        return NULL;
+    profile = gst_omx_h264_utils_get_profile_from_enum (param.eProfile);
+    if (!profile) {
+      g_assert_not_reached ();
+      gst_caps_unref (caps);
+      return NULL;
     }
 
     switch (param.eLevel) {
