@@ -169,7 +169,7 @@ gst_omx_h265_enc_class_init (GstOMXH265EncClass * klass)
 
   gst_element_class_set_static_metadata (element_class,
       "OpenMAX H.265 Video Encoder",
-      "Codec/Encoder/Video",
+      "Codec/Encoder/Video/Hardware",
       "Encode H.265 video streams",
       "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
 
@@ -494,19 +494,6 @@ gst_omx_h265_enc_set_format (GstOMXVideoEnc * enc, GstOMXPort * port,
     gst_caps_unref (peercaps);
   }
 
-  /* Change default profile to main-10 if input is 10 bits */
-  if (profile == OMX_VIDEO_HEVCProfileUnknown) {
-    GstVideoFormat format;
-
-    format = state->info.finfo->format;
-    if (format == GST_VIDEO_FORMAT_NV12_10LE32 ||
-        format == GST_VIDEO_FORMAT_NV16_10LE32) {
-      GST_DEBUG_OBJECT (self,
-          "Set profile to main-10 as input is a 10 bits format");
-      profile = OMX_VIDEO_HEVCProfileMain10;
-    }
-  }
-
   if (profile != OMX_VIDEO_HEVCProfileUnknown
       || level != OMX_VIDEO_HEVCLevelUnknown) {
     /* OMX provides 2 API to set the profile and level. We try using the
@@ -556,28 +543,11 @@ gst_omx_h265_enc_get_caps (GstOMXVideoEnc * enc, GstOMXPort * port,
       "alignment", G_TYPE_STRING, "au", NULL);
 
   if (err == OMX_ErrorNone) {
-    switch (param.eProfile) {
-      case OMX_VIDEO_HEVCProfileMain:
-        profile = "main";
-        break;
-      case OMX_VIDEO_HEVCProfileMain10:
-        profile = "main-10";
-        break;
-#ifdef USE_OMX_TARGET_ZYNQ_USCALE_PLUS
-      case OMX_ALG_VIDEO_HEVCProfileMainStill:
-        profile = "main-still-picture";
-        break;
-      case OMX_ALG_VIDEO_HEVCProfileMain422:
-        profile = "main-422";
-        break;
-      case OMX_ALG_VIDEO_HEVCProfileMain422_10:
-        profile = "main-422-10";
-        break;
-#endif
-      default:
-        g_assert_not_reached ();
-        gst_caps_unref (caps);
-        return NULL;
+    profile = gst_omx_h265_utils_get_profile_from_enum (param.eProfile);
+    if (!profile) {
+      g_assert_not_reached ();
+      gst_caps_unref (caps);
+      return NULL;
     }
 
     switch (param.eLevel) {
